<?php
/**
 * EMetroTile
 * @author Syakur Rahman<shaqman2004@yahoo.com>
 * @copyright Copyright &copy; Syakur Rahman 2013-
 * @copyright Drew Greenwell for MetroJs <http://www.drewgreenwell.com/projects/metrojs>
 * @copyright Tim Holman for TileJs <https://github.com/tholman/tileJs>
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @version 0.1 beta

 * A widget to display metrotiles as seen in Windows 8 operating system.
 * This widget is basically a wrapper for MetroJs < http://www.drewgreenwell.com/projects/metrojs >
 * and TileJs < https://github.com/tholman/tileJs > combined.
 * MetroJs is used to create the tile animation while TileJs is to give the tilting effects.

  How to use:
  1.	Download and extract EMetroTile in your extension directory.
  2.	Call the widget. Following is how array data should be organized.
		[
		*	//Group
		*	['title'=>'title', 'tiles'=>[
		*		// Groupcontent
		*		['content'=>'html_content,
		*			'url'=>'url',
		*			'style'=>'horizontal,vertical,standard,huge',
		*			'position'=>'top,bottom',
		 *			'liveTileOptions'=>array() // as seen in documentation: http://www.drewgreenwell.com/projects/metrojs#liveTileOptions
		*		]
		*	]
		* ]

  Example:
  $this->widget('ext.emetrotile.EMetroTile', array(
	'Tiles'=>array(
				array('title'=>'Test Title', 'tiles'=>array(
					array('content'=>array('test1-a','test1-b'), 'liveTileOptions'=>array('data-speed'=>750, 'data-delay'=>3000,'data-stack'=>true)),
					array('content'=>'test2', 'position'=>'bottom'),
					array('content'=>'test4', 'position'=>'bottom'),
					array('content'=>'Blog', 'style'=>'vertical', 'url'=>'http://blog.expressthisout.com'),
					array('content'=>'test3', 'style'=>'horizontal'),
					array('content'=>'test5', 'position'=>'bottom'),
					array('content'=>'test6', 'position'=>'top'),

				))
		)
  ));

  3. Done

 * Notes:
 * This version is using v 0.9 and
 * TileJs
 *
 * History:
 * 30/1/2013  - Initial release.
 *
  <- ************************************************************************************************ */

/**
 * 
 */
class EMetroTile extends CWidget {
	public $HWidth = 10;
	public $VWidth = 10;
	public $SingleTileSize = 110;

	/* [
	*	//Group
	*	['title'=>'title', 'tiles'=>[
	*		// Groupcontent
	*		['content'=>'html_content,
	*			'url'=>'url',
	*			'style'=>'horizontal,vertical,standard,huge',
	*			'position'=>'top,bottom',
	 *			'liveTileOptions'=>array() // as seen in documentation: http://www.drewgreenwell.com/projects/metrojs#liveTileOptions
	*		]
	*	]
	* ]
	*/
	public $Tiles = array();

	private $_baseUrl = "";
	private $_defaultUrl = "#";
	private $_styleOptions = array('standard', 'horizontal', 'vertical'); /** TODO: add huge **/
	private $_positionOptions = array('top', 'bottom');

	public function run() {
		$this->_validateDataArray($this->Tiles);

		$this->_registerClientScripts();
		$this->_createMarkUp();
	}

	private function _validateDataArray($arr)
	{
		// Check whether given data is an array or not empty.
		if(!is_array($arr))
		{
			throw new CException("Tiles should be a valid array.");
		}
		if(count($arr)<1)
		{
			throw new CException("Tiles should not be an empty array.");
		}

		// Validate each group data.
		foreach($arr as &$group)
		{
			// Check whether given tile data is an array or not empty
			if(!is_array($group['tiles']))
			{
				throw new CException("Tiles data should be a valid array.");
			}
			if(count($group['tiles'])<1)
			{
				throw new CException("Tiles data should not be an empty array.");
			}

			// trim string properties.
			foreach($group as &$property)
			{
				if(is_string($property))
				{
					$property = trim($property);
				}
			}

			// Set Default values
			if(!isset($group['title']))
			{
				$group['title'] = "";
			}

			// Validate tile contents
			foreach($group['tiles'] as &$tile)
			{
				if(isset($tile['liveTileOptions']))
				{
					if(!is_array($tile['liveTileOptions']))
						throw new CException("liveTileOptions should be an array.");
					else {
						if(!isset($tile['liveTileOptions']['data-stack']))
						{
							$tile['liveTileOptions']['data-stack'] = true;
						}
					}

				}

				//trim string properties and validate values
				foreach($tile as $key=>&$property)
				{
					if(is_string($property))
					{
						$property = trim($property);
					}
				}

				// Set default values
				if(!isset($tile['url']))
				{
					$tile['url']=$this->_defaultUrl;
				}
				if(!isset($tile['style']))
				{
					$tile['style'] = $this->_styleOptions[0];
				}
				if(!isset($tile['position']))
				{
					$tile['position'] = $this->_positionOptions[0];
				}

				if(!in_array($tile['style'], $this->_styleOptions))
				{
					throw new CException("Tile style should either be huge, horizontal, vertical or standard.");
				}

				if(!in_array($tile['position'], $this->_positionOptions))
				{
					throw new CException("Tile position should either be top or bottom.");
				}
			}
		}

		//sort array and put bottom values behind while moving vertical values forward
		foreach($arr as &$group)
		{
			for($i=0;$i<count($group['tiles']);$i++)
			{
				if($group['tiles'][$i]['position']=='bottom')
				{
					$group['tiles'][] = $group['tiles'][$i];
					unset($group['tiles'][$i]);
				}
			}

			$group['tiles'] = array_values($group['tiles']);

			for($i=0;$i<count($group['tiles']);$i++)
			{
				if($group['tiles'][$i]['style']=='vertical')
				{
					$tmp = $group['tiles'][$i];
					unset($group['tiles'][$i]);
					array_unshift($group['tiles'], $tmp);
				}
			}
		}

		$this->Tiles = $arr;
		return true;
	}

	private function _genRandomNumber() {
		return substr(md5(rand(100, 999)), 4, 8);
	}

	private function _registerClientScripts() {
		$resources = dirname(__FILE__).DIRECTORY_SEPARATOR."assets";
		$this->_baseUrl = Yii::app()->assetManager->publish($resources);

		// register the files
		Yii::app()->clientScript->registerScriptFile($this->_baseUrl.'/js/tileJs' . (YII_DEBUG ? '' : '.min') .'.js', CClientScript::POS_END);
		Yii::app()->clientScript->registerScriptFile($this->_baseUrl.'/js/MetroJs' . (YII_DEBUG ? '' : '.min') .'.js', CClientScript::POS_HEAD);
		Yii::app()->clientScript->registerCssFile($this->_baseUrl.'/css/MetroJs' . (YII_DEBUG ? '' : '.min') .'.css');
//		Yii::app()->clientScript->registerCssFile($this->_baseUrl.'/css/style.css');

		// Register css
		Yii::app()->clientScript->registerCss("emt-title", "
			#metro-array #emt-title {
				border-bottom: thin solid #ccf;
				font-weight: bold;
				margin-bottom: ".floor($this->VWidth / 2)."px;
				width: ". (($this->SingleTileSize*2)-($this->VWidth * 2)) ."px !important;
			}"
		);
		Yii::app()->clientScript->registerCss("emt-container", "
			#metro-array {
				display: inline-block;
			}"
		);
		Yii::app()->clientScript->registerCss("emt-tile","
			.metro-tile:hover {
				text-decoration: none;
				background: #4e4eff; /* for non-css3 browsers */
				background: -moz-radial-gradient(bottom right, #4e4eff,  #8a8aff); /* for firefox 3.6+ */
				background: -webkit-gradient(radial, bottom right, from(#4e4eff), to(#8a8aff)); /* Safari 4+, Chrome */
				background: -webkit-radial-gradient(bottom right, #4e4eff, #8a8aff); /* Chrome 10+, Safari 5.1+, iOS 5+ */
				background: -o-radial-gradient(bottom right, #4e4eff,#8a8aff); /* Opera 11.10+ */
				background: radial-gradient(bottom right, #4e4eff,#8a8aff); /* W3C */

				font-weight: bold;
			}

			.metro-tile {
				cursor: pointer;
				display:block;
				float:left;
				//background-color: deepSkyBlue;
				background: #4e4eff; /* for non-css3 browsers */
				background: -moz-radial-gradient(top left, #4e4eff,  #8a8aff); /* for firefox 3.6+ */
				background: -webkit-gradient(radial, top left, from(#4e4eff), to(#8a8aff)); /* Safari 4+, Chrome */
				background: -webkit-radial-gradient(top left, #4e4eff, #8a8aff); /* Chrome 10+, Safari 5.1+, iOS 5+ */
				background: -o-radial-gradient(top left, #4e4eff,#8a8aff); /* Opera 11.10+ */
				background: radial-gradient(top left, #4e4eff,#8a8aff); /* W3C */

				-moz-box-shadow: 3px 3px 3px #6d6b6b;/*FF 3.5+*/
				-webkit-box-shadow: 3px 3px 3px #6d6b6b;/*Saf3-4, Chrome, iOS 4.0.2-4.2, Android 2.3+*/
				-ms-filter: 'progid:DXImageTransform.Microsoft.Shadow(Strength=3, Direction=135, Color=#6d6b6b)';/*IE 8*/
				box-shadow: 3px 3px 3px #6d6b6b;
				filter: progid:DXImageTransform.Microsoft.Shadow(Strength=3, Direction=135, Color=#6d6b6b);/*IE 5.5-7*/

				width: " . $this->SingleTileSize . "px;
				height: " . $this->SingleTileSize . "px;
				color: #fff;
				margin: 0px;
			}
			.emt-tall {
				height: " . (($this->SingleTileSize * 2) + $this->VWidth) . "px;
			}

			.emt-hspace {
				margin-left: " . $this->HWidth ."px;
			}

			.emt-vspace {
				margin-top: " . $this->VWidth ."px;
			}
			
			.emt-wide {
				width: " . (($this->SingleTileSize * 2) + $this->HWidth) . "px;
			}
			"
		);
		Yii::app()->clientScript->registerCss("emt-tile-content","
			.emt-content {
			}"
		);
		Yii::app()->clientScript->registerCss("emt-tile-group-default", "
			.metro-array-group {
				height: ". (($this->SingleTileSize*2)+($this->VWidth)+30) ."px !important;
				float: left;
				margin-left: 10px;
			}"
		);
	}

	private function _createMarkUp() {
		echo '<div id="metro-array">';
		foreach($this->Tiles as $group)
		{
			$content = "";
			if(strlen($group['title'])>0)
			{
				$content .= '<div id="emt-title">'.$group['title'].'</div>';
			} else {
				$content .= '<div id="emt-title" style="border:none;">&nbsp;</div>';
			}

			$asFirstTile['top'] = true;
			$asFirstTile['bottom'] = true;
			if($group['tiles'][0]['style'] == 'vertical')
				$asFirstTile['bottom'] = !true;
			$width = 0;
			foreach($group['tiles'] as $tile)
			{
				$class = "metro-tile";
				$style = "";
				switch($tile['position'])
				{
					case "top":
					{
						if($asFirstTile['top'])
						{
							$asFirstTile['top'] = false;
						} else {
							$class .= " emt-hspace ";
							$width += $this->HWidth;
						}
						break;
					}
					case "bottom":
					{
						if($asFirstTile['bottom'])
						{
							$asFirstTile['bottom'] = false;
						} else {
							$class .= " emt-hspace ";
							$width += $this->HWidth;
						}

						$class .= " emt-vspace ";

						break;
					}
				}
				
				switch($tile['style'])
				{
					case "horizontal":
					{
						$class.=" emt-wide ";
						break;
					}
					case "vertical": 
					{
						$class.= " emt-tall ";
						$class = str_replace(array(" emt-vspace "), "",$class);
						break;
					}
				}

				// determine width of group
				if($tile['position'] == "top")
				{
					switch($tile['style'])
					{
						case "huge":
						case "horizontal": $width += ($this->SingleTileSize*2) + $this->HWidth; break;
						case "standard":
						case "vertical": $width += ($this->SingleTileSize); break;
					}
				}

				// Prepare content and ignore if liveTilesOptions are not specified
				$tileContent = "";
				$liveTileOptions = "";
				if(is_array($tile['content']))
				{
					if(isset($tile['liveTileOptions']) && count($tile['liveTileOptions'])>0)
					{
						if(isset($tile['liveTileOptions']['animate']))
						{
							unset($tile['liveTileOptions']['animate']);
						}

						foreach($tile['content'] as $c)
						{
							$options = array();
							if(isset($tile['liveTileOptions']['data-mode']))
							{
								if($tile['liveTileOptions']['data-mode'] == 'flip')
								{
									$options=array("class"=>"metro-tile");
									$style .= " background: #FFF; ";
								}
							}
							$tileContent .= CHtml::tag("div", $options,$c);
						}

						foreach($tile['liveTileOptions'] as $k=>$l)
						{
							$liveTileOptions .= " $k=\"$l\" ";
						}

						Yii::app()->clientScript->registerScript("emt-register-liveTile", '
							$(".live-tile").liveTile();
							'
						, CClientScript::POS_READY);

						$class .= " live-tile ";

					} else {
						$tileContent = CHtml::tag("div", array(),array_shift(array_values($tile['content'])));
					}
				} else {
					$tileContent = CHtml::tag("div", array(),$tile['content']);
				}

				$content .= '<a href="'.$tile['url'].'"><div class="'.$class.'" '.$liveTileOptions.' style="'.$style.'">'.$tileContent.'</div></a>';
//				$content .= '<div class="metro-tile '.$class.'" '.$liveTileOptions.'>'.$tileContent.'</div>';
			}
			$content = '<div class="metro-array-group" style="width: '.$width.'px;">'.$content.'</div>';
			echo $content;
		}
	}

}

?>
